package provaintermedia;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Lorenzo Masetti
 * @version 1.0
 */

public class AstaServer {
  private ServerSocket sSock;

  /** <p>tabella hash che contiene associati agli identificativi dei clienti
   * i loro OutputClient per mandare i messaggi</p>
   */
  private Hashtable clients = new Hashtable();

  /** <p>tabella hash che contiene associato ad ogni articolo
   * il suo banditore</p>
   */
  private Hashtable aste = new Hashtable();

  /** <p>tabella hash che contiene l'articolo per il quale un
   * cliente  stato interrogato</p>
   */
  private Hashtable clientiinterrogati = new Hashtable();

  /** <p>serve per assegnare un codice univoco ad ogni articolo */
  private int codicecorrente = 0;

  /**
   * Crea  una nuova istanza di <code>AstaServer</code>.
   *
   * @param sock  <code>ServerSocket</code>
   */
  public AstaServer(ServerSocket sock) {
    sSock = sock;
  }

  /**
   * <p>Costruisce il server</p>
   * @param port la porta dove girare
   * @exception IOException se si ha un errore nel creare il socket
   */
  public AstaServer(int port) throws IOException {
    System.out.println("Viene creato il server");
    sSock = new ServerSocket(port);
  }

  /**
   * <p>Fa partire il server e lo mette in attesa di connessioni
   * che richiedono il login</p>
   */
  public void start() {
    while (true) {
      try {
        Socket cSock = sSock.accept();
        new Thread(new StartConnection(cSock, this)).start();
      }
      catch (IOException e) {
        System.err.println("Impossibile accettare la connessione");
      }
    }
  }

  /**
   * @return true se l'utente  connesso, false altrimenti
   */
  private boolean connected(IdUtente utente) {
    return (clients.containsKey(utente));
  }

  /**
   * <p>Effettua il login di un utente</p>
   * @param utente l'utente di cui si vuole fare il login
   * @param outp l'OutputClient corrispondente al cliente
   * @return true se il login ha avuto successo
   */
  public boolean login(IdUtente utente, OutputClient outp) {
    synchronized (clients) {
      if (connected(utente)) {
        System.out.println("L'utente " + utente + " esiste gi!");
        return false;
        // in realt questo non dovrebbe mai succedere dato che possono esistere
        // pi utenti con lo stesso nome purch siano diversi l'host o la port
      }

      // inserisce il gestore dell'output nella tabella dei clienti
      clients.put(utente, outp);
      System.out.println("\n\nSi connette " + utente.toString());
      return true;
    }
  }

  /**
   * Esegue il logout di un utente
   *
   * @param utente <code>IdUtente</code>
   */
  public void logout(IdUtente utente) {
    System.out.println("Si disconnette " + utente.toString());
    synchronized (clients) {
      OutputClient oC = (OutputClient) clients.get(utente);
      oC.stop();
      clients.remove(utente);
      Banditore b = (Banditore) clientiinterrogati.get(utente);
      if (b != null) {
        b.rifiuto(utente);
        clientiinterrogati.remove(utente);
      }

    }
  }

  /**
   * Invia un messaggio all'utente indicato.
   * @param to  <code>IdUtente</code> destinatario
   * @param msg il <code>Messaggio</code> da inviare
   * @return true se l'utente esiste. false altrimenti
   */
  public boolean sendMessage(IdUtente to, Messaggio msg) {
    System.out.println("Mando a " + to + " il seguente messaggio:\n" + msg);

    synchronized (clients) {
      OutputClient oC = (OutputClient) clients.get(to);
      if (oC == null) {
        return false; // se l'utente si  disconnesso
      }
      oC.add(msg);
      return true;
    }
  }

  /**
   * Metodo chiamato dalla classe InputClient per gestire messaggi di tipo
   * ARTICOLO, RIALZO e RIFIUTO
   * @param utente  <code>IdUtente</code> (mittente)
   * @param m  <code>Messaggio</code> il messaggio inviato
   */
  public void manageMessage(IdUtente utente, Messaggio m) {
    System.out.println("E' arrivato un messaggio di " + utente + ":\n" + m);
    if (m.getCodice() == Messaggio.ARTICOLO) {
      lanciaAsta(utente, m);
    }
    else {
      msgAsta(utente, m);
    }
  }

  /**
   * Metodo chiamato per creare una nuova asta.
   *
   * @param venditore un valore <code>IdUtente</code> contenente il venditore
   * @param m un <code>Messaggio</code> contenente l'articolo e il prezzo
   */
  private void lanciaAsta(IdUtente venditore, Messaggio m) {
    System.out.println("\nInizia l'asta");
    System.out.println(venditore + " vende il seguente articolo:");
    Articolo daVendere = (Articolo) m.getMessaggio();
    daVendere.setCodice(++codicecorrente); // assegna un codice univoco

    System.out.println(daVendere.getCodice() + " " + daVendere.getDescrizione());
    System.out.println("Prezzo iniziale: " + m.getPrezzo());
    HashSet compratori;
    synchronized (clients) {
      compratori = new HashSet(clients.keySet());
      // crea l'insieme dei possibili compratori.
      // inzialmente contiene tutti i client
    }
    compratori.remove(venditore); // escluso il venditore

    Banditore nuovobanditore = new Banditore(venditore, compratori, daVendere,
                                             m.getPrezzo(), this);
    synchronized (aste) {
      aste.put(daVendere, nuovobanditore);
      // inserisce il banditore nella tabella delle aste in esecuzione
    }
    new Thread(nuovobanditore).start(); // parte il Thread dell'asta
  }

  /**
   * Metodo richiamato per gestire messaggi
   * di tipo RIALZO o RIFIUTO
   * @param id un <code>IdUtente</code> mittente
   * @param m il <code>Messaggio</code>
   */
  private void msgAsta(IdUtente id, Messaggio m) {
    Banditore b = (Banditore) clientiinterrogati.get(id);
    // b  il banditore dell'asta per cui l'utente  stato interrogato
    synchronized (clientiinterrogati) {
      clientiinterrogati.remove(id);
    }
    System.out.println("E' arrivato un messaggio d'asta dall'utente " + id +
                       " lo levo da quelli da aspettare");
    if (m.getCodice() == Messaggio.RIALZO) {
      b.rialzo(id, m.getPrezzo());
    }
    else { // RIFIUTO
      b.rifiuto(id);
    }
    synchronized (id) {
      id.notify();
    }
    // risveglia le aste che stavano aspettando per interrogare questo utente
  }

  /**
   * Metodo chiamato dal banditore per notificare la fine di un'asta
   * @param a l' <code>Articolo</code> al quale l'asta si riferisce
   */
  public void fineAsta(Articolo a) {
    System.out.println("E' finita l'asta per l'articolo " + a.getDescrizione() +
                       " (" + a.getCodice() + ")");
    synchronized (aste) {
      aste.remove(a); // toglie il banditore dalla tabella delle aste
    }
  }

  /**
   * Metodo richiamato dalla classe Banditore per inviare un messaggio
   * di offerta a un cliente. Inserisce il cliente nella tabella dei
   * clienti interrogati
   *
   * @param to l'<code>IdUtente</code> destinatario
   * @param b il <code>Banditore</code> che richiama il metodo
   * @param prezzo (<code>float</code>) il prezzo d'offerta
   * @return true se il messaggio  stato inviato correttamente
   *         cio se il cliente  ancora collegato
   */
  public boolean inviaMessaggioOfferta(IdUtente to, Banditore b, float prezzo) {
    synchronized (clientiinterrogati) {
      System.out.println("Inserisco " + to +
                         " nella tabella dei clienti che devono rispondere");
      clientiinterrogati.put(to, b);
      boolean inviato = sendMessage(to,
                                    Messaggio.messaggioOfferta(b.getArticolo(),
          prezzo));
      if (!inviato) {
        clientiinterrogati.remove(to);
      }
      return inviato;
    }
  }

  /**
   * Per determinare se un utente  stato interrogato per un'offerta
   * ma non ha ancora risposto
   *
   * @param utente un <code>IdUtente</code>
   * @return true se non ha risposto, false se non ha offerte pendenti
   */
  public boolean impegnato(IdUtente utente) {
    synchronized (clientiinterrogati) {
      return (clientiinterrogati.containsKey(utente));
    }
  }

  /**
   * Metodo main, fa partire il sever
   *
   * @param args  Prende un solo argomento, la porta dove mettersi in ascolto
   * @exception IOException se non riesce a creare il socket
   */
  public static void main(String[] args) throws IOException {
    if (args.length != 1) {
      throw new IllegalArgumentException("Sintassi: AstaServer <port>");
    }

    try {
      int port = Integer.parseInt(args[0]);
      new AstaServer(port).start();
    }
    catch (NumberFormatException e) {
      throw new IllegalArgumentException("Sintassi: AstaServer <port>");
    }
  }

}

class IdUtente
    implements Serializable {
  private InetAddress host;
  private int port;
  private String name;

  public IdUtente(String name, InetAddress host, int port) {
    this.name = name;
    this.host = host;
    this.port = port;
  }

  public InetAddress getHost() {
    return host;
  }

  public int getPort() {
    return port;
  }

  public String getName() {
    return name;
  }

  public String toString() {
    return name + "@" + host.getCanonicalHostName() + ":" + port;
  }

  public boolean equals(Object other) {
    if (! (other instanceof IdUtente)) {
      return false;
    }
    IdUtente oth = (IdUtente) other;
    return ( (oth.host.equals(host)) && (oth.port == port) &&
            (oth.name.equals(name)));
  }
}

class StartConnection
    implements Runnable {
  private Socket clientSock;
  private AstaServer server;

  public StartConnection(Socket sock, AstaServer server) {
    this.clientSock = sock;
    this.server = server;
  }

  public void run() {
    try {
      ObjectOutputStream oos = new ObjectOutputStream(clientSock.getOutputStream());
      ObjectInputStream ois = new ObjectInputStream(clientSock.getInputStream());
      Object msg = ois.readObject();
      if (msg instanceof Messaggio) {
        Messaggio mess = (Messaggio) msg;
        System.out.println(mess);
        if (mess.getCodice() == Messaggio.LOGIN) {
          IdUtente id = new IdUtente( (String) mess.getMessaggio(),
                                     clientSock.getInetAddress(),
                                     clientSock.getPort());
          OutputClient oC = new OutputClient(oos);
          InputClient iC = new InputClient(id, ois, server);
          if (!server.login(id, oC)) {
            System.out.println("Login impossibile!");
          }
          else {
            new Thread(oC).start();
            new Thread(iC).start();
          }
        }
      }
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (ClassNotFoundException e) {
      e.printStackTrace();
    }

  }
}

class Banditore
    implements Runnable {

  /** insieme dei partecipanti all'asta */
  private Set partecipanti;

  /** Tabella delle offerte di ogni cliente */
  private Hashtable offerte = new Hashtable();

  private IdUtente venditore;
  private Articolo daVendere;
  private AstaServer server;
  private float prezzo;

  public Banditore(IdUtente venditore, Set partecipanti, Articolo daVendere,
                   float prezzoIniz, AstaServer server) {
    this.venditore = venditore;
    this.partecipanti = partecipanti;
    this.daVendere = daVendere;
    this.server = server;
    this.prezzo = prezzoIniz;
  }

  Articolo getArticolo() {
    return daVendere;
  }

  public void run() {
    IdUtente compratore = null;
    int giro = 1;
    while (giro <= 2) {
      int rialzi = 0;
      // si notifica l'offerta a tutti i partecipanti
      Iterator it = partecipanti.iterator();
      while (it.hasNext()) {
        IdUtente interrogato = (IdUtente) it.next();
        if (interrogato == compratore) {
          continue; // non si manda il messaggio a chi ha fatto l'offerta piu' alta
        }

        if (server.impegnato(interrogato)) {
          // se quello che si deve interpellare  impgnato ci si mette
          // in attesa sulla coda associata all'utente.
          synchronized (interrogato) {
            try {
              interrogato.wait();
            }
            catch (InterruptedException e) {
              e.printStackTrace();
            }
          }
        }
        if (! (server.inviaMessaggioOfferta(interrogato, this, prezzo))) {
          System.out.println("BANDITORE dell'articolo " + daVendere +
                             ": Ignoro " + interrogato +
                             " che probabilmente si  disconnesso");
          continue;
          //  se nel frattempo si  staccato lo
          // ignora
        }
        synchronized (offerte) {
          try {
            offerte.wait();
          }
          catch (InterruptedException e) {
            e.printStackTrace();
          }
        }
        float prezzoofferto = ( (Float) offerte.get(interrogato)).floatValue();
        if (prezzoofferto > prezzo) {
          prezzo = prezzoofferto;
          rialzi++;
          compratore = interrogato;
        }

      }
      if (rialzi <= 1) {
        giro++;
      }
    }

    if (compratore == null) {
      System.out.println("BANDITORE: Asta per " + daVendere +
                         " senza compratore");
      server.sendMessage(venditore,
                         Messaggio.messaggioDeserta(daVendere, prezzo));
    }
    else {
      System.out.println("BANDITORE: Vince l'asta per " + daVendere +
                         " l'utente " + compratore);
      server.sendMessage(compratore,
                         Messaggio.messaggioAggiudicato(daVendere, prezzo));
      server.sendMessage(venditore,
                         Messaggio.messaggioFineOK(daVendere,
          prezzo));
    }
    server.fineAsta(daVendere);
  }

  public void rialzo(IdUtente utente, float prezzo) {
    synchronized (offerte) {
      offerte.put(utente, new Float(prezzo));
      offerte.notify();
    }
  }

  public void rifiuto(IdUtente utente) {
    synchronized (offerte) {
      offerte.put(utente, new Float( -1));
      offerte.notify();
    }

  }
}
