package provaintermedia;

import java.io.*;
import java.net.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class Cliente extends JFrame {
  Socket sCliente;
  ObjectInputStream ois;
  ObjectOutputStream oos;

  public static void main(String[] argv)
  throws IOException, UnknownHostException {
    if (argv.length != 3) {
      throw new IllegalArgumentException("Sintassi: Cliente <host> <port> <nome>");
    }
    int port;
    try {
      port = Integer.parseInt(argv[1]);
    } catch (NumberFormatException e) {
      throw new IllegalArgumentException("Sintassi: Cliente <host> <port> <nome>");
    }
    Cliente c = new Cliente( argv[0] , port , argv[2] );
    c.setVisible(true);
  }

  public Cliente(String host , int port , String nome )
      throws IOException , UnknownHostException {
    super(nome+" - Cliente asta...");
    setupPanel();
    System.out.println("Connessione al server...");
    sCliente = new Socket( host , port );
    System.out.println("Connessione effettuata!");
    ois = new ObjectInputStream( sCliente.getInputStream() );
    oos = new ObjectOutputStream( sCliente.getOutputStream());
    System.out.println("Login...");
    oos.writeObject(Messaggio.messaggioLogin(nome));
    System.out.println("Login Ok!");
    Thread t = new Thread( new ClientReadThread( this, sCliente , ois , oos ));
    t.start();
    pack();
    setVisible(true);
  }

  public void setupPanel() {
    JPanel jp = new JPanel();
    JButton jb = new JButton( "Articolo all'asta" );
    jb.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        sendArticolo();
      }
    });
    jp.add( jb );
    getContentPane().add(jp);
  }

  public void sendArticolo() {
    synchronized (sCliente) {
      dialogArticolo dg = new dialogArticolo(this);
      dg.setVisible(true);
      if (dg.getState()) {
        try {
          Articolo a = dg.getArticolo();
          float prezzo = dg.getPrezzo();
          Messaggio msg = Messaggio.messaggioArticolo(a, prezzo);
          oos.writeObject(msg);
        }
        catch (Exception e) {
          JOptionPane.showMessageDialog(this, e.getMessage(),
                                        "Errore:" + e.getClass().getName(),
                                        JOptionPane.ERROR_MESSAGE);
        }
      }
    }
  }

}

class ClientReadThread implements Runnable {

  JFrame owner;
  Socket sCliente;
  ObjectInputStream ois;
  ObjectOutputStream oos;

  public ClientReadThread(JFrame owner , Socket sCliente , ObjectInputStream ois, ObjectOutputStream oos) {
    this.owner = owner;
    this.sCliente = sCliente;
    this.ois = ois;
    this.oos = oos;
  }

  public void run() {
    while (true) {
      try {
        Messaggio msg = (Messaggio) ois.readObject();
        if (msg.getCodice() == Messaggio.OFFERTA) {
          Articolo a = (Articolo) msg.getMessaggio();
          float prezzo = msg.getPrezzo();
          trattaOfferta( a , prezzo );
        }
        if (msg.getCodice() == Messaggio.DESERTA ) {
          Articolo a = (Articolo) msg.getMessaggio();
          JOptionPane.showMessageDialog(owner , "Asta deserta: "+a.getDescrizione()+" ( "+a.getCodice()+" ) ", "Asta deserta..." , JOptionPane.INFORMATION_MESSAGE);
        }
        if (msg.getCodice() == Messaggio.AGGIUDICATO ) {
          Articolo a = (Articolo) msg.getMessaggio();
          JOptionPane.showMessageDialog(owner , "Articolo aggiudicato: "+a.getDescrizione()+" ( "+a.getCodice()+" ) "+" Euro: "+msg.getPrezzo(), "Aggiudicato articolo...", JOptionPane.INFORMATION_MESSAGE);
        }
        if (msg.getCodice() == Messaggio.FINEOK ) {
          Articolo a = (Articolo) msg.getMessaggio();
          JOptionPane.showMessageDialog(owner , "Articolo venduto: "+a.getDescrizione()+" ( "+a.getCodice()+" ) "+" Euro: "+msg.getPrezzo(), "Aggiudicato articolo...", JOptionPane.INFORMATION_MESSAGE);
        }
      }
      catch (IOException e) {
        e.printStackTrace();
        System.exit(-1);
      }
      catch (ClassNotFoundException e) {
        e.printStackTrace();
        System.exit(-1);
      }
    }
  }

  public void trattaOfferta(Articolo a , float prezzo ) {
    float nuovoPrezzo = prezzo;
    synchronized ( sCliente ) {
      String descrizione = a.getDescrizione() + " ( " + a.getCodice() + " ) ";
      offertaDialog od = new offertaDialog(owner, descrizione, prezzo);
      boolean flag = true;
      try {
        do {
          od.setVisible(true);
          if (od.getState()) {
            nuovoPrezzo = od.getPrezzo();
            if (nuovoPrezzo > prezzo) {
              oos.writeObject(Messaggio.messaggioRialzo(nuovoPrezzo));
              flag = false;
            }
            else {
              JOptionPane.showMessageDialog(owner,
                                            "Inserire un prezzo piu' alto!",
                                            "Errore", JOptionPane.ERROR_MESSAGE);
              flag = true;
            }
          }
          else {
            oos.writeObject(Messaggio.messaggioRifiuto());
            flag = false;
          }
        }
        while (flag);
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
  }
}


class articoloPanel extends JPanel {
  GridBagLayout gbl;
  GridBagConstraints gbc;
  JTextField codice;
  JTextField descrizione;
  JTextField prezzo;

  public articoloPanel() {
    super();
    gbl = new GridBagLayout();
    gbc = new GridBagConstraints();
    gbc.insets = new java.awt.Insets(5,5,5,5);
    setLayout(gbl);
    codice = new JTextField(10);
    descrizione = new JTextField(50);
    prezzo = new JTextField(10);
//    addNewLine( "Codice: " , codice);
    addNewLine( "Descrizione: " , descrizione);
    addNewLine( "Prezzo: " , prezzo );
  }

  void addNewLine( String label , JTextField jtf ) {
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
    gbc.gridwidth = 1;
    gbc.anchor = GridBagConstraints.WEST;
    JLabel jlabel = new JLabel(label);
    gbl.setConstraints(jlabel , gbc);
    add(jlabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER; //end row
    gbl.setConstraints(jtf,gbc);
    add(jtf);
  }

  Articolo getArticolo()
      throws NumberFormatException {
    //String strCodice = codice.getText();
    String strDescrizione = descrizione.getText();
    Articolo toReturn = new Articolo( -1 , strDescrizione );
    return toReturn;
  }

  float getPrezzo()
      throws NumberFormatException {
    String strPrezzo = prezzo.getText();
    return Float.parseFloat(strPrezzo);
  }
}

class dialogArticolo extends JDialog {
  articoloPanel ap;
  boolean state;

  public dialogArticolo( JFrame owner ) {
    super( owner , "Articolo all'asta..." , true );
    getContentPane().setLayout(new BorderLayout());
    state = false;
    ap = new articoloPanel();
    getContentPane().add(ap,BorderLayout.CENTER);
    JPanel jp = new JPanel();
    JButton jb = new JButton("Ok");
    jb.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        OnOk();
      }
    });
    jp.add(jb);
    jb = new JButton("Annulla");
    jb.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        OnCancel();
      }
    });
    jp.add(jb);
    getContentPane().add(jp , BorderLayout.SOUTH);
    pack();
  }

  public void OnOk() {
    state = true;
    setVisible(false);
  }

  public void OnCancel() {
    state = false;
    setVisible(false);
  }

  public boolean getState() {
    return state;
  }

  public Articolo getArticolo() throws NumberFormatException {
    if (state) {
      return ap.getArticolo();
    }
    return null;
  }

  public float getPrezzo() throws NumberFormatException {
    return ap.getPrezzo();
  }
}

class offertaDialog extends JDialog {

  boolean state = false;
  offertaPanel op;

  public offertaDialog( JFrame owner , String descrizione , float prezzo ) {
    super( owner , "Offerta..." , true );
    op = new offertaPanel( descrizione , prezzo );
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(op,BorderLayout.CENTER);
    JPanel bp = new JPanel();
    JButton button;
    button = new JButton("Rialza");
    button.addActionListener( new ActionListener() {
      public void actionPerformed( ActionEvent e ) {
        OnRialza();
      }
    });
    bp.add( button );
    button = new JButton("Ignora");
    button.addActionListener( new ActionListener() {
      public void actionPerformed( ActionEvent e ) {
        OnPassa();
      }
    });
    bp.add( button );
    getContentPane().add(bp , BorderLayout.SOUTH);
    setResizable(false);
    pack();
  }

  public void OnRialza() {
    state = true;
    setVisible(false);
  }

  public void OnPassa() {
    state = false;
    setVisible(false);
  }

  public float getPrezzo() {
    return op.getPrezzo();
  }

  public boolean getState() {
    return state;
  }

}

class offertaPanel extends JPanel {
  GridBagLayout gbl;
  GridBagConstraints gbc;
  JTextField prezzo;
  float offerta;

  public offertaPanel(String descrizione , float offerta ) {
    super();
    gbl = new GridBagLayout();
    gbc = new GridBagConstraints();
    gbc.insets = new java.awt.Insets(5,5,5,5);
    setLayout(gbl);
    prezzo = new JTextField(offerta+"",10);
    addNewLine( "Articolo: " , new JLabel(descrizione));
    addNewLine( "Prezzo: " , prezzo );
  }

  void addNewLine( String label , JComponent jtf ) {
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
    gbc.gridwidth = 1;
    gbc.anchor = GridBagConstraints.WEST;
    JLabel jlabel = new JLabel(label);
    gbl.setConstraints(jlabel , gbc);
    add(jlabel);
    gbc.gridwidth = GridBagConstraints.REMAINDER; //end row
    gbl.setConstraints(jtf,gbc);
    add(jtf);
  }

  float getPrezzo()
      throws NumberFormatException {
    String strPrezzo = prezzo.getText();
    return Float.parseFloat(strPrezzo);
  }
}
