package provaintermedia;

import java.io.*;

/**
 * <p>Title: InputClient</p>
 * <p>Description: Classe Runnable per un thread che ascolta un utente dell'asta</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author Lorenzo Masetti
 * @version 1.0
 */

public class InputClient
    implements Runnable {
  private IdUtente utente;
  private ObjectInputStream ois;
  private AstaServer server;
  private boolean attivo = true;

    /**
     * Crea un nuovo <code>InputClient</code>
     *
     * @param utente  <code>IdUtente</code> l'utente da ascoltare
     * @param ois  l'<code>ObjectInputStream</code> associato al socket del cliente
     * @param server l'<code>AstaServer</code> a cui riferirsi
     */
    public InputClient(IdUtente utente, ObjectInputStream ois, AstaServer server) {
    this.utente = utente;
    this.ois = ois;
    this.server = server;
  }

    /**
     * Metodo implementato da Runnable che mette il thread in ascolto per
     * nuovi messaggi
     *
     */
    public void run() {
    while (attivo) {
      try {
        Object m = ois.readObject();
        if (m instanceof Messaggio) {
          server.manageMessage( utente, (Messaggio) m);
        }
      }
      catch (EOFException e) {
        attivo = false;
        server.logout(utente);
      }
      catch (IOException e) {
        e.printStackTrace();
        attivo = false;
      }
      catch (ClassNotFoundException e) {
        e.printStackTrace();
        attivo = false;
      }

    }
  }

    /**
     * Per fermare il thread costruito a partire da questa classe
     *
     */
    public void stop() {
    attivo = false;
  }

}




